﻿// MZpack 3 Pro API demo
// Copyright © 2018 MZpack Co.
// www.mzpack.pro
#if APISAMPLE
using System;
using System.Collections.Generic;
using NinjaTrader.Data;
using MZpack;
using MZpack.NT8;
using System.ComponentModel.DataAnnotations;

namespace NinjaTrader.NinjaScript.Strategies
{
    /// <summary>
    /// MZpack custom strategy SAMPLE #1. 
    /// It uses StrategyBigTradeIndicator and StrategyVolumeProfileIndicator classes.
    /// </summary>
    /// <summary>
    /// Strategy logic description.
    /// </summary>
    /// <summary>
    /// 1. Calculate bar range on bar close; 
    /// </summary>
    /// <summary>
    /// 2. If bar range greater or equal to 24 start building of developing profile; 
    /// </summary>
    /// <summary>
    /// 3. If profile range less or equal to 20 or profile range >= 50 cancel building profile;
    /// </summary>
    /// <summary>
    /// 3.1 Calculate big trades delta inside profile;
    /// </summary>
    /// <summary>
    /// 3.2 Enter position: enter SHORT if Delta is negative and bigTradeDelta is positive; enter LONG if Delta is positive and bigTradeDelta is negative;
    /// </summary>
    /// <summary>
    /// See installation folder for the source code.
    /// </summary>
    public class MZpackCustomStrategy1 : MZpackStrategyBase
    {
        CustomVolumeProfileIndicator customVP;
        CustomBigTradeIndicator customBigTrade;
        IVolumeProfile profile; // Current volume profile

        string signalLong, signalShort;

        /// <summary>
        /// Create indicators here and return them in the list as a result.
        /// </summary>
        /// <returns>Indicators list</returns>
        public override List<TickIndicator> CreateIndicators()
        {
            List<TickIndicator> indicators = new List<TickIndicator>(); // Initialize new indicators list

            // Create CustomVolumeProfileIndicator instance with default settings
            // See CustomVolumeProfileIndicator class
            customVP = new CustomVolumeProfileIndicator(this)
            {
                // Override defaults if required
                ProfileCreation = ProfileCreation.Custom,
                ProfileMode = ProfileMode.BuySell
            };
            indicators.Add(customVP);  // Add "Custom Volume Profile" indicator to the list

            // Create CustomBigTradeIndicator instance with default settings
            // See CustomBigTradeIndicator class
            customBigTrade = new CustomBigTradeIndicator(this)
            {
                // Override defaults if required
                OrderType = OrderFlowOrderType.Market,
                BigtradeFilter = Strategy_TradeVolume,
                IcebergFilter = 10000,  // Ignore iceberg orders
                DomPressureFilter = 10000,  // Ignore DOM pressure
                FilterLogic = TradeFilterLogic.OR,
                AggressionFilter = Strategy_AggressionFilter,
                MarkerType = TradeMarkerType.Box,
                MinShapeExtent = 0.3,
                MaxShapeExtent = 1.5,
                UseBigTradeAlert = false
            };
            indicators.Add(customBigTrade);

            return indicators;
        }

        protected override void OnStateChange()
        {
            base.OnStateChange(); // Base OnStateChange() call is required

            try
            {
                lock (Sync) // Sync state handler
                {
                    if (State == State.SetDefaults)
                    {
                        Calculate = Calculate.OnBarClose; // OnBarClose strategy
                        EntriesPerDirection = 1;
                    }
                    else if (State == State.Configure)
                    {
                        // Configure indicators
                        customVP.StackedProfileCreation1 = ProfileCreation.None;  // Set Stacked Profile #1 creation
                        customVP.StackedProfileCreation2 = ProfileCreation.None;  // Set Stacked Profile #2 creation
                        customVP.StackedProfileCreation3 = ProfileCreation.None;  // Set Stacked Profile #3 creation
                    }
                }
            }
            catch (Exception ex)
            {
                HandleThrowException(ex); // Handle errors. HandleThrowException() just prints and rises exception
            }
        }

        protected override void OnBarUpdate()
        {
            base.OnBarUpdate();  // Base OnBarUpdate() call is required

            if (BarsInProgress == 0)
            {
                try
                {
                    lock (Sync) // Sync event handler
                    {
                        profile?.RefreshModel(RefreshModelArgs.None);  // Refresh data model for current profile if exists

                        int barRange = (int)((High[0] - Low[0]) / TickSize); // Calculate bar range in ticks

                        // Create new profile and exit position if exists
                        if (profile == null && barRange >= 24)
                        {
                            // Exit long if exists
                            if (!string.IsNullOrEmpty(signalLong))
                            {
                                ExitLong("Cover", signalLong);
                                signalLong = null;
                            }
                            // Exit short if exists
                            if (!string.IsNullOrEmpty(signalShort))
                            {
                                ExitShort("Cover.", signalShort);
                                signalShort = null;
                            }

                            profile = customVP.AddProfile();  // Add new profile 
                            profile.AssignRange(CurrentBar, CurrentBar); // This makes the profile starting from closed bar, but not from next bar
                            profile.IsDeveloping = true;  // Make it developing
                        }
                        // Cancel building profile and enter position
                        else if (profile != null && (barRange <= 20 || profile.RangeTicks >= 50))
                        {
                            // Enter position
                            if (Bars != null && Bars.Count >= BarsRequiredToTrade)
                            {
                                // Calculate big trades delta inside profile
                                long bigTradeDelta = 0;
                                for (int i = profile.BeginBarIdx; i <= profile.EndBarIdx; i++)
                                {
                                    if (customBigTrade.Trades.ContainsKey(i))
                                    {
                                        // For each trade in bar with index i
                                        foreach (ITrade trade in customBigTrade.Trades[i])
                                        {
                                            if (trade.Side == TradeSide.Bid)
                                                bigTradeDelta -= trade.Volume;  // Sell trade
                                            else
                                            if (trade.Side == TradeSide.Ask)
                                                bigTradeDelta += trade.Volume;  // Buy trade
                                        }
                                    }
                                }

                                if (profile.Delta < 0 && bigTradeDelta > 0)
                                {
                                    signalShort = $"VP delta:{profile.Delta}, T delta:{bigTradeDelta}";
                                    SetProfitTarget(signalShort, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                                    SetStopLoss(signalShort, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                                    // Enter short if Delta is negative and bigTradeDelta is positive
                                    EnterShort(signalShort);
                                }
                                else
                                if (profile.Delta > 0 && bigTradeDelta < 0)
                                {
                                    signalLong = $"VP delta:{profile.Delta}, T delta:{bigTradeDelta}";
                                    SetProfitTarget(signalLong, CalculationMode.Ticks, Strategy_TakeProfitTicks);
                                    SetStopLoss(signalLong, CalculationMode.Ticks, Strategy_StopLossTicks, false);
                                    // Enter long if Delta is positive and bigTradeDelta is negative
                                    EnterLong(signalLong);
                                }
                            }

                            // Cancel building of developing profile
                            profile.IsDeveloping = false;
                            profile = null;
                        }
                    }
                }
                catch (Exception ex)
                {
                    HandleThrowException(ex); // Handle errors. HandleThrowException() just logs and rises exception
                }
            }
        }


        [Display(Name = "Take profit, ticks", GroupName = "00. Strategy", Order = 1, Description = "")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_TakeProfitTicks
        { get; set; } = 96;

        // Maximal stop loss
        [Display(Name = "Stop loss, ticks", GroupName = "00. Strategy", Order = 1, Description = "")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_StopLossTicks
        { get; set; } = 32;

        /// <summary>
        /// Volume Profile Indicator attribute
        /// </summary>
        [Display(Name = "Trade volume", GroupName = "01. Filtering", Order = 1, Description = "Trade volume filter")]
        [Range(0, long.MaxValue)]
        [NinjaScriptProperty]
        public long Strategy_TradeVolume
        { get; set; } = 400;

        /// <summary>
        /// Volume Profile Indicator attribute
        /// </summary>
        [Display(Name = "Aggression/Sweep, ticks", GroupName = "01. Filtering", Order = 20, Description = "Aggression or sweep filter in ticks")]
        [Range(1, long.MaxValue)]
        [NinjaScriptProperty]
        public int Strategy_AggressionFilter
        { get; set; } = 4;
    }


    /// <summary>
    /// StrategyVolumeProfileIndicator is a base class for any Volume Profile indicator based on MZpack Pro API 
    /// 'public' is required by UI 
    /// </summary>
    public class CustomVolumeProfileIndicator : StrategyVolumeProfileIndicator
    {
        public CustomVolumeProfileIndicator(MZpackStrategyBase strategy)
            : base(strategy)
        {
        }

        /// <summary>
        /// Override indicator name
        /// </summary>
        /// <returns>Indicator name for UI</returns>
        public override string IndicatorName() => @"Custom Volume Profile";  // Override indicator name

        /// <summary>
        /// Use indicator with Tick replay only
        /// </summary>
        /// <returns>True if indicator processes historical data with Tick replay turned off</returns>
        public override bool NoTickReplayHistoricalSupport() => false; 
    }


    /// <summary>
    /// StrategyBigTradeIndicator is a base class for any Big Trade indicator based on MZpack Pro API 
    /// 'public' is required by UI 
    /// </summary>
    public class CustomBigTradeIndicator : StrategyBigTradeIndicator
    {
        public CustomBigTradeIndicator(MZpackStrategyBase strategy)
            : base(strategy)
        {
        }

        /// <summary>
        /// Override indicator name
        /// </summary>
        /// <returns>Indicator name for UI</returns>
        public override string IndicatorName() => @"Custom BigTrade Indicator";

        /// <summary>
        /// Use indicator with Tick replay only
        /// </summary>
        /// <returns>True if indicator processes historical data with Tick replay turned off</returns>
        public override bool NoTickReplayHistoricalSupport() => false; 
    }
}
#endif